@@ -3,26 +3,22 @@ package Dancer2::Plugin::FormValidator;
 use 5.24.0;
 
 use Dancer2::Plugin;
+use Module::Load qw(autoload);
 use Dancer2::Core::Hook;
+use Dancer2::Plugin::FormValidator::Validator;
 use Dancer2::Plugin::FormValidator::Config;
-use Dancer2::Plugin::FormValidator::Registry;
-use Dancer2::Plugin::FormValidator::Processor;
-use Storable qw(dclone);
-use Hash::Util qw(lock_hashref);
-use Module::Load;
-use Types::Standard qw(InstanceOf HashRef);
+use Types::Standard qw(InstanceOf HashRef ArrayRef);
 
-our $VERSION = '0.71';
+our $VERSION = '0.80';
 
 # Global var for saving last success validation valid input.
 my $valid_input;
 
 plugin_keywords qw(validate validated errors);
 
-has config_obj => (
+has config_validator => (
     is       => 'ro',
     isa      => InstanceOf['Dancer2::Plugin::FormValidator::Config'],
-    required => 1,
     builder  => sub {
         return Dancer2::Plugin::FormValidator::Config->new(
             config => shift->config,
@@ -30,16 +26,7 @@ has config_obj => (
     }
 );
 
-has plugin_deferred => (
-    is       => 'ro',
-    isa      => InstanceOf ['Dancer2::Plugin::Deferred'],
-    required => 1,
-    builder  => sub {
-        return shift->app->with_plugin('Dancer2::Plugin::Deferred');
-    }
-);
-
-has extensions => (
+has config_extensions => (
     is       => 'ro',
     isa      => HashRef,
     default  => sub {
@@ -47,7 +34,92 @@ has extensions => (
     }
 );
 
+has extensions => (
+    is       => 'ro',
+    isa      => ArrayRef,
+    builder  => sub {
+        my $self = shift;
+
+        my @extensions = map {
+            my $extension = $self->config_extensions->{$_}->{provider};
+            autoload $extension;
+
+            $extension->new(
+                plugin => $self,
+                config => $self->config_extensions->{$_},
+            );
+        } keys %{ $self->config_extensions };
+
+        return \@extensions;
+    }
+);
+
+has plugin_deferred => (
+    is       => 'ro',
+    isa      => InstanceOf ['Dancer2::Plugin::Deferred'],
+    builder  => sub {
+        return shift->app->with_plugin('Dancer2::Plugin::Deferred');
+    }
+);
+
 sub BUILD {
+    shift->_register_hooks;
+    return;
+}
+
+sub validate {
+    my ($self, %args) = @_;
+
+    # We need to unset value of this global var.
+    undef $valid_input;
+
+    # Now works with arguments.
+    my $profile = %args{profile};
+    my $input   = %args{input} // $self->dsl->body_parameters->as_hashref_mixed;
+    my $lang    = %args{lang};
+
+    if (defined $lang) {
+        $self->_validator_language($lang);
+    }
+
+    my $validator = Dancer2::Plugin::FormValidator::Validator->new(
+        config            => $self->config_validator,
+        input             => $input,
+        extensions        => $self->extensions,
+        validator_profile => $profile,
+    );
+
+    my $result = $validator->validate;
+
+    if ($result->success != 1) {
+        $self->plugin_deferred->deferred(
+            $self->config_validator->session_namespace,
+            {
+                messages => $result->messages,
+                old      => $input,
+            },
+        );
+
+        return undef;
+    }
+    else {
+        $valid_input = $result->valid;
+        return $valid_input;
+    }
+}
+
+sub validated {
+    my $valid = $valid_input;
+    undef $valid_input;
+
+    return $valid;
+}
+
+sub errors {
+    return shift->_get_deferred->{messages};
+}
+
+sub _register_hooks {
     my $self = shift;
 
     $self->app->add_hook(
@@ -58,7 +130,7 @@ sub BUILD {
                 my $errors = {};
                 my $old    = {};
 
-                if (my $deferred = $tokens->{deferred}->{$self->config_obj->session_namespace}) {
+                if (my $deferred = $tokens->{deferred}->{$self->config_validator->session_namespace}) {
                     $errors = delete $deferred->{messages};
                     $old    = delete $deferred->{old};
                 }
@@ -70,118 +142,18 @@ sub BUILD {
             },
         )
     );
-}
 
-sub validate {
-    my ($self, %params) = @_;
-
-    # We need to unset value of this global var.
-    $valid_input = undef;
-
-    # Now works with arguments.
-    my $profile = %params{profile};
-    my $input   = %params{input} // $self->dsl->body_parameters->as_hashref_mixed;
-    my $lang    = %params{lang};
-
-    if (defined $lang) {
-        $self->_validator_language($lang);
-    }
-
-    my $result = $self->_validate($input, $profile);
-
-    if ($result->success) {
-        $valid_input = $result->valid;
-        return $valid_input;
-    }
-
-    return undef;
-}
-
-sub validated {
-    my $valid    = $valid_input;
-    $valid_input = undef;
-
-    return $valid;
-}
-
-sub errors {
-    return shift->_get_deferred->{messages};
-}
-
-
-sub _validator_language {
-    shift->config_obj->language(shift);
     return;
 }
 
-sub _validate {
-    my ($self, $input, $validator_profile) = @_;
-
-    if (ref $input ne 'HASH') {
-        Carp::croak "Input data should be a hash reference\n";
-    }
-
-    my $role = 'Dancer2::Plugin::FormValidator::Role::Profile';
-    if (not $validator_profile->does($role)) {
-        my $name = $validator_profile->meta->name;
-        Carp::croak "$name should implement $role\n";
-    }
-
-    my $processor = Dancer2::Plugin::FormValidator::Processor->new(
-        input             => $self->_clone_and_lock_input($input),
-        config            => $self->config_obj,
-        registry          => $self->_registry,
-        validator_profile => $validator_profile,
-    );
-
-    my $result = $processor->result;
-
-    if ($result->success != 1) {
-        $self->plugin_deferred->deferred(
-            $self->config_obj->session_namespace,
-            {
-                messages => $result->messages,
-                old      => $input,
-            },
-        );
-    }
-
-    return $result;
-}
-
-sub _clone_and_lock_input {
-    # Copy input to work with isolated HashRef.
-    my $input = dclone($_[1]);
-
-    # Lock input to prevent accidental modifying.
-    return lock_hashref($input);
-}
-
-sub _registry {
-    my $self = shift;
-
-    # First build extensions.
-    my @extensions = map
-    {
-        my $extension = $self->extensions->{$_}->{provider};
-        autoload $extension;
-
-        $extension->new(
-            plugin => $self,
-            config => $self->extensions->{$_},
-        );
-    }
-        keys %{ $self->extensions };
-
-    return Dancer2::Plugin::FormValidator::Registry->new(
-        extensions => \@extensions,
-    );
+sub _validator_language {
+    shift->config_validator->language(shift);
+    return;
 }
 
 sub _get_deferred {
     my $self = shift;
-
-    return $self->plugin_deferred->deferred($self->config_obj->session_namespace);
+    return $self->plugin_deferred->deferred($self->config_validator->session_namespace);
 }
 
 1;
@@ -199,7 +171,7 @@ Dancer2::Plugin::FormValidator - neat and easy to start form validation plugin f
 
 =head1 VERSION
 
-version 0.71
+version 0.80
 
 =head1 SYNOPSIS
 
@@ -245,7 +217,7 @@ The html result could be like:
 =begin html
 
 <p>
-  <img alt="Screenshot register form" src="https://raw.githubusercontent.com/AlexP007/dancer2-plugin-formvalidator/645b8e8d7c91739c7618a2088841f93f47c53495/assets/screenshot_register.png" width="500px">
+  <img alt="Screenshot register form" src="https://raw.githubusercontent.com/AlexP007/dancer2-plugin-formvalidator/main/assets/screenshot_register.png" width="500px">
 </p>
 
 =end html
@@ -357,7 +329,7 @@ Now you can validate POST parameters in your controller:
 
 =head2 Template
 
-In you template you have access to: $errors - this is HashRef with parameters names as keys
+In you template you have access to: $errors - this is HashRef with fields names as keys
 and error messages as ArrayRef values and $old - contains old input values.
 
 Template app/register:
@@ -433,13 +405,38 @@ Template app/register:
                     ...
     ...
 
+=head2 session
+
+=head3 namespace
+
+Session storage key where this module stores data, like: errors or old vars.
+
+=head2 messages
+
+=head3 language
+
+Default language for error messages.
+
+=head3 ucfirst
+
+Apply ucfirst function to messages or not.
+
+=head3 validators
+
+Key => values, where key is validator name and value is messages
+dictionary for different languages.
+
+=head2 extensions
+
+Key => values, where key is extension short name and values is its configuration.
+
 =head1 DSL KEYWORDS
 
 =head3 validate
 
     validate(Hash $params): HashRef|undef
 
-Accept params as hash:
+Accept arguments as hash:
 
     (
         profile => Object implementing Dancer2::Plugin::FormValidator::Role::Profile # required
@@ -451,6 +448,15 @@ Profile is required, input and lang is optional.
 
 Returns valid input HashRef if validation succeed, otherwise returns undef.
 
+    ### You can use HashRef returned from validate.
+
+    if (my $valid_hash_ref = validate profile => RegisterForm->new) {
+        # Success, data is valid.
+    }
+
+
+    ### Or more declarative approach with validated keyword.
+
     if (validate profile => RegisterForm->new) {
         # Success, data is valid.
         my $valid_hash_ref = validated;
@@ -459,7 +465,7 @@ Returns valid input HashRef if validation succeed, otherwise returns undef.
     }
     else {
         # Error, data is invalid.
-        my $errors = errors;
+        my $errors = errors; # errors keyword returns error messages.
 
         # Redirect or show errors...
     }
@@ -489,30 +495,46 @@ Returns HashRef[ArrayRef] if validation failed.
 
 Validates that field B<exists> and one of the listed: (yes on 1).
 
-=head3 alpha
+    field => [ qw(accepted) ]
 
-Validate that string only contain of alphabetic utf8 symbols, i.e. /^[[:alpha:]]+$/.
+=head3 alpha:encoding=ascii
 
-=head3 alpha_ascii
+Validate that string only contain of alphabetic symbols.
+By default encoding is ascii, i.e /^[[:alpha:]]+$/a.
 
-Validate that string only contain of latin alphabetic ascii symbols, i.e. /^[[:alpha:]]+$/a.
+    field => [ qw(alpha) ]
+
+To set encoding to unicode you need to pass 'u' argument:
+
+    field => [ qw(alpha:u) ]
+
+Then the validation rule will be /^[[:alpha:]]+$/.
 
 =head3 alpha_num
 
-Validate that string only contain of alphabetic utf8 symbols, underscore and numbers 0-9, i.e. /^\w+$/.
+Validate that string only contain of alphabetic symbols, underscore and numbers 0-9.
+By default encoding is ascii, i.e. /^\w+$/a.
 
-=head3 alpha_num_ascii
+    field => [ qw(alpha_num) ]
 
-Validate that string only contain of latin alphabetic ascii symbols, underscore and numbers 0-9, i.e. /^\w+$/a.
+To set encoding to unicode you need to pass 'u' argument:
+
+    field => [ qw(alpha_num:u) ]
+
+Rule will be /^\w+$/.
 
 =head3 email
 
 Validate that field is valid email(rfc822).
 
+    field => [ qw(email) ]
+
 =head3 email_dns
 
 Validate that field is valid email(rfc822) and dns exists.
 
+    field => [ qw(email_dns) ]
+
 =head3 enum:value1,value2
 
 Validate that field is one of listed values.
@@ -523,6 +545,8 @@ Validate that field is one of listed values.
 
 Validate that field is integer.
 
+    field => [ qw(integer) ]
+
 =head3 length_max:num
 
 Validate that string length <= num.
@@ -551,11 +575,20 @@ Validate that field is number >= num.
 
 Validate that field is number.
 
+    field => [ qw(numeric) ]
+
 =head3 required
 
 Validate that field exists and not empty string.
 
-=head3 same
+    field => [ qw(required) ]
+
+=head3 same:field
+
+Validate that field is exact value as another.
+
+    field_1 => [ qw(required) ]
+    field_2 => [ qw(required same:field_1) ]
 
 =head1 CUSTOM MESSAGES
 
@@ -681,9 +714,6 @@ L<Dancer2::Plugin::FormValidator::Extension::DBIC|https://metacpan.org/pod/Dance
 
 =over 4
 
-=item *
-Document all config field with explanation.
-
 =item *
 Document all Roles and HashRef structures.
 
